@@ -4,8 +4,11 @@ package Coat::Persistent;
 use Coat;
 use Coat::Meta;
 use Coat::Persistent::Meta;
+use Coat::Persistent::Constraint;
 use Carp 'confess';
 
+use Data::Dumper;
+
 # Low-level helpers
 use Digest::MD5 qw(md5_base64);
 use Scalar::Util qw(blessed looks_like_number);
@@ -16,11 +19,15 @@ use DBI;
 use DBIx::Sequence;
 use SQL::Abstract;
 
+# Constants
+use constant CP_ENTRY_NEW => 0;
+use constant CP_ENTRY_EXISTS => 1;
+
 # Module meta-data
 use vars qw($VERSION @EXPORT $AUTHORITY);
 use base qw(Exporter);
 
-$VERSION   = '0.100';
+$VERSION   = '0.210';
 $AUTHORITY = 'cpan:SUKRIA';
 @EXPORT    = qw(has_p has_one has_many);
 
@@ -29,7 +36,6 @@ my $sql_abstract = SQL::Abstract->new;
 
 # configuration place-holders
 my $MAPPINGS    = {};
-my $CONSTRAINTS = {};
 
 # static accessors
 sub mappings { $MAPPINGS }
@@ -49,6 +55,17 @@ sub cache {
     undef;
 }
 
+# Access to the constraint meta data for the current class
+sub has_unique_constraint {
+    my ($class, $attr) = @_;
+    $class->has_constraint($attr, 'unique');
+}
+
+sub has_constraint {
+    my ($class, $attr, $constraint) = @_;
+    Coat::Persistent::Constraint->get_constraint($constraint, $class, $attr) || 0;
+}
+
 sub enable_cache {
     my ($class, %options) = @_;
     $class = '!default' if $class eq 'Coat::Persistent';
@@ -73,6 +90,31 @@ sub disable_cache {
     undef $MAPPINGS->{'!cache'}{$class};
 }
 
+# A singleton that stores the driver/module mappings
+# The ones here are default drivers that are known to be compliant
+# with Coat::Persistent.
+# Any DBI driver should work though.
+my $drivers = {
+    csv    => 'DBI:CSV',
+    mysql  => 'dbi:mysql',
+    sqlite => 'dbi:SQLite',
+};
+sub drivers { $drivers }
+
+# Accessor to a driver
+sub get_driver {
+    my ($class, $driver) = @_;
+    confess "driver needed" unless $driver;
+    return $class->drivers->{$driver};
+}
+
+# This lets you add the DBI driver you want to use
+sub add_driver {
+    my ($class, $driver, $module) = @_;
+    confess "driver and module needed" unless $driver and $module;
+    $class->drivers->{$driver} = $module;
+}
+
 # This is the configration stuff, you basically bind a class to
 # a DBI driver
 sub map_to_dbi {
@@ -82,11 +124,9 @@ sub map_to_dbi {
     # if map_to_dbi is called from Coat::Persistent, this is the default dbh
     $class = '!default' if $class eq 'Coat::Persistent';
 
-    my $drivers = {
-        mysql => 'dbi:mysql',
-        csv   => 'DBI:CSV',
-    };
-    confess "No such driver : $driver"
+    my $drivers = Coat::Persistent->drivers;
+
+    confess "No such driver : $driver, please register the driver first with add_driver()"
       unless exists $drivers->{$driver};
 
     # the csv driver needs to load the appropriate DBD module
@@ -110,6 +150,16 @@ sub map_to_dbi {
     _create_dbix_sequence_tables($MAPPINGS->{'!dbh'}{$class});
 }
 
+# This is used if you already have a dbh instead of creating one with 
+# map_to_dbi 
+sub set_dbh {
+    my ($class, $dbh) = @_;
+    confess "Cannot set an undefined dbh" unless defined $dbh;
+
+    $class = '!default' if $class eq 'Coat::Persistent';
+    $MAPPINGS->{'!dbh'}{$class} = $dbh;
+    _create_dbix_sequence_tables($MAPPINGS->{'!dbh'}{$class});
+}
 
 # This is done to wrap the original Coat::has method so we can
 # generate finders for each attribute declared
@@ -129,11 +179,29 @@ sub has_p {
     confess "package main called has_p" if $caller eq 'main';
 
     # unique field ?
-    $CONSTRAINTS->{'!unique'}{$caller}{$attr} = $options{unique} || 0;
-    # syntax check ?
-    $CONSTRAINTS->{'!syntax'}{$caller}{$attr} = $options{syntax} || undef;
+    if ($options{'unique'}) {
+        Coat::Persistent::Constraint->add_constraint('unique', $caller, $attr, 1);
+    }
+    
+    # specific storage type ?
+    if ($options{'store_as'}) {
+        # We need bi-directional coercion for this "store_as" feature ...
+        my $storage_type = Coat::Types::find_type_constraint($options{'store_as'});
+        confess "Unknown type \"".$options{'store_as'}."\" for storage" 
+            unless defined $storage_type;
+        confess "No coercion defined for storage type \"".$options{'store_as'}."\""
+            unless $storage_type->has_coercion;
+
+        my $type = Coat::Types::find_type_constraint($options{isa});
+        confess "No cercion for attribute type : \"".$options{isa}."\"" 
+            unless $type->has_coercion;
+
+        Coat::Persistent::Constraint->add_constraint('store_as', $caller, $attr, $options{'store_as'});
+        $options{coerce} = 1;
+    }
 
     Coat::has( $attr, ( '!caller' => $caller, %options ) );
+    Coat::Persistent::Meta->attribute($caller, $attr);
 
     # find_by_
     my $sub_find_by = sub {
@@ -427,19 +495,21 @@ sub find_by_sql {
         # if any rows, let's process them
         if (@$rows) {
             # we have to find out which fields are real attributes
-            my $class_attr = Coat::Meta->all_attributes( $class );
-            my @attrs = keys %$class_attr;
-            
-            # from the columns selected, where are real attributes and virtual ones?
+            my @attrs = Coat::Persistent::Meta->linearized_attributes( $class );
             my $lc = new List::Compare(\@attrs, [keys %{ $rows->[0] }]);
             my @given_attr   = $lc->get_intersection;
             my @virtual_attr = $lc->get_symdiff;
 
             # create the object with attributes, and set virtual ones
             foreach my $r (@$rows) {
+
                 my $obj = $class->new(map { ($_ => $r->{$_}) } @given_attr);
                 $obj->init_on_find();
-                $obj->{$_} = $r->{$_} for @virtual_attr;
+                foreach my $field (@virtual_attr) {
+                    $obj->{$field} = $r->{$field};
+                }
+
+                $obj->{_db_state} = CP_ENTRY_EXISTS;
                 push @objects, $obj;
             }
         }
@@ -463,23 +533,21 @@ sub find_by_sql {
 sub init_on_find {
 }
 
+sub BUILD {
+    my ($self) = @_;
+    $self->{_db_state} = CP_ENTRY_NEW;
+}
+
 sub validate {
     my ($self, @args) = @_;
     my $class = ref($self);
     my $table_name  = Coat::Persistent::Meta->table_name($class);
     my $primary_key = Coat::Persistent::Meta->primary_key($class);
     
-    foreach my $attr (keys %{ Coat::Meta->all_attributes($class)} ) {
-        # checking for syntax validation
-        if (defined $CONSTRAINTS->{'!syntax'}{$class}{$attr}) {
-            my $regexp = $CONSTRAINTS->{'!syntax'}{$class}{$attr};
-            confess "Value \"".$self->$attr."\" for attribute \"$attr\" is not valid"
-                unless $self->$attr =~ /$regexp/;
-        }
+    foreach my $attr (Coat::Persistent::Meta->linearized_attributes($class) ) {
         
         # checking for unique attributes on inserting (new objects)
-        if ((! defined $self->$primary_key) && 
-            $CONSTRAINTS->{'!unique'}{$class}{$attr}) {
+        if ($class->has_unique_constraint($attr)) {
             # look for other instances that already have that attribute
             my @items = $class->find(["$attr = ?", $self->$attr]);
             confess "Value ".$self->$attr." violates unique constraint "
@@ -487,7 +555,6 @@ sub validate {
                 if @items;
         }
     }
-
 }
 
 sub delete {
@@ -534,6 +601,39 @@ sub create {
     }
 }
 
+# This will return the value as to be stored in the underlying database
+# Most of the time it's just the value of the atrtribute, but it can 
+# be different if a 'store_as' type is defined.
+sub get_storage_value_for {
+    my ($self, $attr_name) = @_;
+    my $class = ref $self;
+
+    my $attr = Coat::Meta->attribute($class, $attr_name);
+
+    if ($attr->{store_as}) {
+        my $storing_type = Coat::Types::find_type_constraint($attr->{store_as});
+        return $storing_type->coerce($self->$attr_name);
+    }
+    else {
+        return $self->$attr_name;
+    }
+}
+
+# Takes a value (taken from the DB) and convert it to the real value for the attribute
+sub get_real_value_for {
+    my ($self, $attr_name, $value) = @_;
+    my $class = ref $self;
+
+    my $attr = Coat::Meta->attribute($class, $attr_name);
+    if ($attr->{store_as}) {
+        my $type = Coat::Types::find_type_constraint($attr->{isa});
+        return $type->coerce($value);
+    }
+    else {
+        return $value;
+    }
+}
+
 # serialize the instance and save it with the mapper defined
 sub save {
     my ($self) = @_;
@@ -546,16 +646,18 @@ sub save {
     confess "Cannot save without a mapping defined for class " . ref $self
       unless defined $dbh;
 
-    # first call validate to check the object is sane
+    # make sure the object is sane
     $self->validate();
 
     # all the attributes of the class
-    my @fields = keys %{ Coat::Meta->all_attributes( ref $self ) };
-    # a hash containing attr/value pairs for the current object.
-    my %values = map { $_ => $self->$_ } @fields;
+    my @fields = Coat::Persistent::Meta->linearized_attributes( ref $self );
+
+    # a hash containing attr/value pairs for the current object
+    my %values = map { $_ => $self->get_storage_value_for($_) } @fields;
+
+    # if not a new object, we have to update
+    if ( $self->_db_state == CP_ENTRY_EXISTS ) {
 
-    # if we have an id, update
-    if ( defined $self->$primary_key ) {
         # generate the SQL
         my ($sql, @values) = $sql_abstract->update(
             $table_name, \%values, { $primary_key => $self->$primary_key});
@@ -565,11 +667,17 @@ sub save {
           or confess "Unable to execute query \"$sql\" : $DBI::errstr";
     }
 
-    # no id, insert with a valid id
+    # new object, insert
     else {
+        # if the id has been touched, trigger an error, that's not possible
+        # with the use of DBIx::Sequence
+        if ($self->{id}) {
+            confess "The id has been set on a newborn object of class ".ref($self).", cannot save, id would change";
+        }
+
         # get our ID from the sequence
         $self->$primary_key( $self->_next_id );
-
+    
         # generate the SQL
         my ($sql, @values) = $sql_abstract->insert(
             $table_name, { %values, $primary_key => $self->$primary_key });
@@ -579,6 +687,8 @@ sub save {
         my $sth = $dbh->prepare($sql);
         $sth->execute( @values )
           or confess "Unable to execute query \"$sql\" : $DBI::errstr";
+
+        $self->{_db_state} = CP_ENTRY_EXISTS;
     }
 
     # if subobjects defined, save them
@@ -652,6 +762,13 @@ sub _next_id {
     return $id;
 }
 
+# Returns a constant describing if the object exists or not
+# already in the underlying DB
+sub _db_state {
+    my ($self) = @_;
+    return $self->{_db_state} ||= CP_ENTRY_NEW;
+}
+
 # DBIx::Sequence needs two tables in the schema,
 # this private function create them if needed.
 sub _create_dbix_sequence_tables($) {
@@ -758,12 +875,58 @@ You can overide those conventions at import time:
 
 =head1 CONFIGURATION
 
+You have two options for setting a database handle to your class. Either you
+already have a dbh an you set it to your class, or you don't and you let
+Coat::Persistent initialize it.
+
+If you already have a database handle, use Coat::Persistent->set_dbh($dbh),
+otherwise, use the DBI mapping explained below.
+
+=head2 Setting an existing database handle
+
+=over 4
+
+=item B<set_dbh($dbh)>
+
+Set the given database handle for the calling class (set it by default if class
+is Coat::Persistent).
+
+=back
+
 =head2 DBI MAPPING
 
 You have to tell Coat::Persistent how to map a class to a DBI driver. You can
 either choose to define a default mapper (in most of the cases this is what
 you want) or define a mapper for a specific class.
 
+In order for your mapping to be possible, the driver you use must be known by
+Coat::Persistent, you can modify its driver mapping matrix if needed.
+
+=over 4
+
+=item B<drivers( )>
+
+Return a hashref representing all the drivers mapped.
+
+  MyClass->drivers;
+
+=item B<get_driver( $name )>
+
+Return the Perl module of the driver defined for the given driver name.
+  
+  MyClass->get_driver( 'mysql' );
+
+=item B<add_driver( $name, $module )>
+
+Add or replace a driver mapping rule. 
+
+  MyClass->add_driver( sqlite => 'dbi:SQLite' );
+
+=back
+
+Then, you can use your driver in mapping rules. Basically, the mapping will
+generate a DBI-E<gt>connect() call.
+
 =over 4 
 
 =item B<Coat::Persistent-E<gt>map_to_dbi $driver, @options >
@@ -901,12 +1064,6 @@ Those methods must be called in class-context.
 
 =over 4 
 
-=item B<find( @conditions, \%options )>
-
-Find operates with three different retrieval approaches:
-
-=over 4
-
 =item I<Find by id>: This can either be a specific id or a list of ids (1, 5,
 6)
 
